﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MvcEdition;

namespace Mvc.Controllers
{
    public class HomeController : Controller
    {
        // GET: Home
        public ActionResult Index()
        {
            Map map = new Map("Map1", 1680, 3256);
            map.MapUnit = GeographyUnit.Meter;
            map.CurrentExtent = new RectangleShape(-8928715.6533264, 4944550.0706076, -8867566.1836358, 4898687.9683396);


            map.MapBackground = new BackgroundLayer(new GeoSolidBrush(GeoColor.StandardColors.Transparent));
            map.MapTools.OverlaySwitcher.Enabled = true;
            map.MapTools.PanZoomBar.IsGlobeButtonEnabled = false;

            map.CustomOverlays.Add(SetupOSMOverlay());
            map.CustomOverlays.Add(SetupZonesOverlay());
            map.CustomOverlays.Add(SetupZipCodeOverlay());
            map.CustomOverlays.Add(SetupMunicipalitiesOverlay());




            //LayerOverlay layerOverlay = new LayerOverlay();
            //layerOverlay.ClientCache = new ClientCache(new TimeSpan(1, 0, 0), "mapClientCache");


            return View(map);
        }

        private Overlay SetupMunicipalitiesOverlay()
        {
            string overlayID = "municipalities", overlayName = "Municipalities", fileName = "Municn.shp", layerKey = "municipalities";

            //Setup the Overlay to hold and display the data.
            LayerOverlay muniOverlay = new LayerOverlay(overlayID, false, TileType.MultipleTile);

            //Set the Overlay's name, which will be displayed in the Overlay Switcher.
            muniOverlay.Name = overlayName;

            //Populate the Layer with data from the shapefile.
            ShapeFileFeatureLayer muniLayer = new ShapeFileFeatureLayer(Server.MapPath("~/App_Data/" + fileName));

            //Display the data from the shapefile based on the type of data.
            muniLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.Transparent, GeoColor.StandardColors.Green);

            //Label Features based on column values in the .dbf file associated with the shapefile, located in the center of the Feature by default for polygons.
            TextStyle textStyle = new TextStyle("Label", new GeoFont("", 10), new GeoSolidBrush(GeoColor.StandardColors.Green));

            muniLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = textStyle;

            //Setup the Layer to apply the Styles used to all ZoomLevels.
            muniLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //Apply the Projection to the Layer. The Projection must be opened before this is done and should be closed afterwards.
            Proj4Projection NAD83toSphMercator = new Proj4Projection(4269, 3857);
            try
            {
                NAD83toSphMercator.Open();
                muniLayer.FeatureSource.Projection = NAD83toSphMercator;
            }
            finally
            {
                NAD83toSphMercator.Close();
            }

            //Add the Layer to the Overlay.
            muniOverlay.Layers.Add(layerKey, muniLayer);

            //Setup caching for the Overlay.

            //Setup the cleint-side caching.
            //muniOverlay.ClientCache = MapClientCache;

            // Setup the server - side caching.
            //muniOverlay.ServerCache = new ServerCache(Server.MapPath(""));
            //muniOverlay.ServerCache.CacheId = MunicipalitiesTileCacheDirectory;

            //Set the Overlay's visibility properties.
            muniOverlay.IsVisible = true;
            muniOverlay.IsVisibleInOverlaySwitcher = true;

            return muniOverlay;
        }

        private Overlay SetupZipCodeOverlay()
        {
            string overlayID = "zipCodes", overlayName = "ZIP Codes", fileName = "Zips.shp", layerKey = "zipCodes";

            //Setup the Overlay to hold and display the data.
            LayerOverlay zipsOverlay = new LayerOverlay(overlayID, false, TileType.MultipleTile);

            //Set the Overlay's name, which will be displayed in the Overlay Switcher.
            zipsOverlay.Name = overlayName;

            //Populate the Layer with data from the shapefile.
            ShapeFileFeatureLayer zipsLayer = new ShapeFileFeatureLayer(Server.MapPath("~/App_Data/" + fileName));

            //Setup Styles determining how the Layer will be displayed.

            //Display the data from the shapefile based on the type of data.
            zipsLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.Transparent, GeoColor.StandardColors.Red);

            //Label Features based on column values in the .dbf file associated with the shapefile, located in the center of the Feature by default for polygons.
            TextStyle textStyle = new TextStyle("ZIP", new GeoFont("", 10), new GeoSolidBrush(GeoColor.StandardColors.Red));

            zipsLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = textStyle;

            //Setup the Layer to apply the Styles used to all ZoomLevels.
            zipsLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //Apply the Projection to the Layer. The Projection must be opened before this is done and should be closed afterwards.
            Proj4Projection NAD83toSphMercator = new Proj4Projection(4269, 3857);

            try
            {
                NAD83toSphMercator.Open();
                zipsLayer.FeatureSource.Projection = NAD83toSphMercator;
            }
            finally
            {
                NAD83toSphMercator.Close();
            }

            //Add the Layer to the Overlay.
            zipsOverlay.Layers.Add(layerKey, zipsLayer);

            //Setup caching for the Overlay.

            //Setup the cleint-side caching.
            //zipsOverlay.ClientCache = MapClientCache;

            //Setup the server-side caching.
            //zipsOverlay.ServerCache = new ServerCache(MapServerTileCacheDirectory);
            //zipsOverlay.ServerCache.CacheId = ZipCodesTileCacheDirectory;

            //Set the Overlay's visibility properties.
            zipsOverlay.IsVisible = true;
            zipsOverlay.IsVisibleInOverlaySwitcher = true;

            return zipsOverlay;
        }

        private Overlay SetupZonesOverlay()
        {
            string overlayID = "zones", overlayName = "Zones", fileName = "Zones.shp", layerKey = "zones";

            LayerOverlay zonesOverlay = new LayerOverlay(overlayID, false, TileType.MultipleTile);

            zonesOverlay.Name = overlayName;

            ShapeFileFeatureLayer zonesLayer = new ShapeFileFeatureLayer(Server.MapPath("~/App_Data/" + fileName));


            zonesLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.Transparent, GeoColor.StandardColors.Blue);

            TextStyle textStyle = new TextStyle("ZONE_NAME", new GeoFont("", 10), new GeoSolidBrush(GeoColor.StandardColors.Blue));

            zonesLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = textStyle;

            zonesLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            Proj4Projection NAD27toSphMercator = new Proj4Projection(4267, 3857);
            try
            {

                NAD27toSphMercator.Open();
                zonesLayer.FeatureSource.Projection = NAD27toSphMercator;
            }
            finally
            {
                NAD27toSphMercator.Close();
            }

            zonesOverlay.Layers.Add(layerKey, zonesLayer);


            //zonesOverlay.ClientCache = MapClientCache;

            //Setup the server-side caching.
            //zonesOverlay.ServerCache = new ServerCache(MapServerTileCacheDirectory);
            //zonesOverlay.ServerCache.CacheId = ZonesTileCacheDirectory;

            zonesOverlay.IsVisible = true;
            zonesOverlay.IsVisibleInOverlaySwitcher = true;

            return zonesOverlay;
        }

        private Overlay SetupOSMOverlay()
        {
            string overlayID = "osmOverlay", overlayName = "OpenStreetMap", layerKey = "osm";

            //Setup the Overlay to hold and display the data.
            LayerOverlay osmOverlay = new LayerOverlay(overlayID, true, TileType.MultipleTile);

            //Set the Overlay's name, which will be displayed in the Overlay Switcher.
            osmOverlay.Name = overlayName;

            //Create the OpenStreetMapLayer.
            OpenStreetMapLayer osmLayer = new OpenStreetMapLayer();

            //Set the caching properties for the Layer.
            //osmLayer.CachePictureFormat = OpenStreetMapLayerPictureFormat.Png32;

            // osmLayer.TileCache = new FileBitmapTileCache(MapServerTileCacheDirectory);
            //osmLayer.TileCache.CacheId = OpenStreetMapsTileCacheDirectory;

            //Add the Layer to the Overlay.
            osmOverlay.Layers.Add(layerKey, osmLayer);

            return osmOverlay;
        }


    }
}